********************************************************************
* BROWPROC.PRG    Contains procedures for Browse routine           *
* Initial version by Bryan Cooper and Monique Verrier              *
* 10/27/1987                                                       *
********************************************************************
* Browset:  This procedure sets up all required memory variables   *
*           for the browse procedures.  It then calls "paint"      *
*           program to paint initial screen.  If a new browse      *
*           window is called by "nextbrow", then browset is re-run,*
*           which resets all variables to the new values.          *
********************************************************************
procedure browset 
  select browse
  store trim(browse->seleprog) to seleprog   && Program run for Selected row
  store browse->quitkey to quitkey           && Key to quit from Browse
  store browse->selekey to selekey           && Key to determine row is selected
  store browse->deleteok to deleteok         && Allows records to be deleted
  store browse->helpkey to helpkey           && Key to determine help is needed
  w.set title to browse->title left  && Puts correct browse title up.
  w.display            
  store trim(browse->field1) + ' '  to field1
  select &select
  go recstart                  && Go to first record to browse
  store 0 to maxrec2           && Maximum number of records on screen, set by paint
  do paint                    && Paints initial screen of data
  goto rec                     && Positions over correct record
  store startrow to row        && Line to highlight on screen
  return 

***********************************************************************
* PAINT  program.  Paints a full screen for the current browse window.*
*        Generally, there will be a "hardcoded" paint screen to make. *
*        If a new browse is added, however, Paint can handle it easily*
*        using a generic macro-substituted loop.                      *
***********************************************************************
procedure paint
clear                       && Clears browse window
select Browse               && Find right "paint" program to run
store 0 to maxrec2         && Maxrec2 determines number of records on screen
select &select             && Selects working area in database
do case
  case browse->paint = " 1"   && Hardcoded first browse for demo
    do while maxrec2 < maxrec .and. .not. eof()
       @ maxrec2,0 say acct_no+'  '+name+'  '+str(prevbal,8,2)+'  '+;
                       str(charges,8,2)+'  '+str(payments,8,2)+'  '+;
                       str(totaldue,8,2)+ iif(delete(),' DEL','    ')
       maxrec2 = maxrec2 + 1
       skip
    enddo
  case browse->paint = " 2"
    do while maxrec2 < maxrec .and. .not. eof()
      @ maxrec2,0 say acct_no+'     '+remarks + ' ' + ;
        salesnum + ' ' + sales->salesper + ' ' + sales->phone+;
        iif(delete(),' DEL','    ')
      maxrec2 = maxrec2 + 1
      skip
    enddo
  case browse->paint = " 3"
    do while maxrec2 < maxrec .and. .not. eof()
      @ maxrec2,0 say sales->salesnum + ' ' + sales->salesper +;
                      ' ' + sales->phone+iif(delete(),' DEL','    ')
      maxrec2 = maxrec2 + 1
      skip
    enddo
  otherwise    && Macro substituted loop just in case!
    do while maxrec2 < maxrec .and. .not. eof()
      @ maxrec2,0 say &field1+ iif(delete(),' DEL','    ')
      maxrec2 = maxrec2 + 1
      skip
    enddo
  endcase
store 0 to row             
return

********************************************************************
* NOTE:  Scrollu, scrolld, scrollpu, scrollpd, and nextbrow are    *
*        fairly complex.  They integrate various memory variables, *
*        and thus keep track of what is on the screen.  You may    *
*        want to skip over them until after using Browse yourself  *
********************************************************************

********************************************************************
* Scrollu:   Scrolls the browse window up one line.                *
*            Uses new Quicksilver 1.1 command "upscroll".          *
*            Scrollu is called after the database has been moved.  *
*            It sets the "rec" variable to correct recno()         *
*            and sets the "row" variable to bottom of screen.      * 
********************************************************************
procedure scrollu              
  store maxrec-1 to row        && Must be at bottom of browse window
  upscroll 1                   && Makes one blank line on screen.
  store recno() to rec         && New record to highlight
  return
  
********************************************************************
* Scrolld:   Scrolls the browse window down one line.              *
*            Uses new Quicksilver 1.1 command "downscroll".        *
*            If more records are shown on the screen, scrolld will *
*            increment "maxrec2".  It sets "row" to 0 since must be*
*            at top of screen, and sets "rec" to correct recno()   *
********************************************************************
procedure scrolld            
  if maxrec2 < maxrec         && Increments count of records on screen
      store maxrec2 + 1 to maxrec2
  endif
  store 0 to row              && Top of browse window
  downscroll 1                && Makes one blank line
  store recno() to rec        && Record to highligt
  return

********************************************************************
* Scrollpu:  This procedure determines whether to scroll the browse*
*            window down one page up.  The "paint" program does   *
*            the actual painting of the screen.                    *
*   NOTE: Will not repaint screen if on first record in database   *
********************************************************************
procedure scrollpu            && Scrolls up 1 page
  store recno() to rec        && Save current place
  skip -row                   && Go HOME first
  skip 1-maxrec               && Now skip back
  if bof()                    && No more records found.
     goto top
  endi
  if rec <> recno()            && paint screen only if record moved
     store recno() to rec
     do paint
     goto rec
  endif
  return

********************************************************************
* Scrollpd:  This procedure determines whether to scroll the browse*
*            window down one page down.  The "paint" program does *
*            the actual painting of the screen.                    *
*   NOTE: Will not repaint screen if on last record in database    *
********************************************************************
procedure scrollpd          
   store recno() to rec       && Save current place
   skip -row                  && Go up to first row on screen
   skip maxrec-1              && Now do actual database skip.
   if recno() = rec .and. .not. eof()  && If at bottom of browse screen,
     do paint                         && do paint now...
     goto rec
   endif
   if eof()
      goto bottom
   endif
   if rec <> recno()          
      store recno() to rec    
      do paint
      goto rec
   endif
   return

********************************************************************
* Nextbrow:  This procedure determines whether to scroll the browse*
*            window left or right.  The browse database can contain*
*            literally dozens of entries, one per browse window.   *
********************************************************************
procedure nextbrow         
select browse
store recno() to oldbrowse
if entered = 4             && Right arrow - go down in Browse.dbf
   skip 1
else
   skip -1                 && Left arrow - go up in Browse.dbf
endif
if browse->database = m->select .and. .not. bof() 
   select &select                   && New BROWSE screen to show.
   store recno() to rec             && Let's save current record and row
   skip -row                        && number for next BROWSE screen.
   store recno() to recstart
   store m->row to startrow
   goto m->rec                      && Go back to original record
   do browset                       && Draws new browse screen.
   return                           && Ready to show new browse.
endif
go oldbrowse                        && No other browse screen, return to start
select &select
return


*********************************************************************
* Getdata: Enters data on the first browse screen, using current    * 
*          row() to determine line for entry.                       *
*          dBaseIII Browse can't perform the mathematical replace   *
*********************************************************************
procedure getdata
   @ ROW(),12 GET ACCOUNTS->NAME PICT '@!'
   @ ROW(),44 GET ACCOUNTS->CHARGES PICT '99999.99'
   @ ROW(),54 GET ACCOUNTS->PAYMENTS PICT '99999.99'
   read
   replace totaldue with prevbal+charges-payments  
   return

*********************************************************************
* Getrmks: Enters data on the second browse screen                  *
*          dBaseIII Browse can't have related fields on the screen. *
*          Getrmks also browses another database if the entry is    *
*          incorrect in the salesnum field!                         *
*********************************************************************
procedure getrmks
   @ ROW(),15 GET ACCOUNTS->REMARKS PICT '@!' 
   @ ROW(),31 GET ACCOUNTS->salesnum
   read
   select sales
   seek accounts->salesnum
   if eof() .or. mod(readkey(),256) = 33 && Control - Home hit
      * Let's do another browse if the salesnum is incorrect
      w.select 11 
      w.use browse
      do browse with ' 1', 1
      w.close 11
      w.select 10  && Go back to original browse 
      replace accounts->salesnum with sales->salesnum
    endif
   select accounts
   return

procedure browhelp
      w.select 12 
      w.use browse
      w.set title to "Browse Help Screen"
      w.display
      w.set size to 17, 10, 23, 60
      text
 Hit enter to edit record.  Use arrow
 keys to scroll up, down, left and right.
 The salesnum field has it's own browse
 based help system.  Try it!

       endtext
 wait
       w.close 12
       w.select 10
return

*** END BROWPROC.PRG
